%{
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "strpool.h"
#include "nametab.h"
#include "tree.h"
#include "genrtl.h"
#include "rtl.h"
#include "scanner.h"
#include "comp.h"
#include "genasm.h"

int yyerror(const char *s);
extern FILE *yyin;

%}

%term   NAME            /* Name */
%term   ICON            /* Integer 123, 'c' */
%term   STRING          /* String "abc" */
%term   PLUS            /* + */
%term   MINUS           /* - */
%term   MUL             /* * */
%term   AND             /* & */
%term   OR              /* | */
%term   XOR             /* ^ */
%term   ASG             /* = */
%term   RELOP           /* LT LE GT GE */
%term   EQUOP           /* EQ NE */
%term   DIVOP           /* DIV MOD */
%term   SHIFTOP         /* LS RS */
%term   ICOP            /* ICR DECR */
%term   LP              /* ( */
%term   RP              /* ) */
%term   LC              /* { */
%term   RC              /* } */
%term   LB              /* [ */
%term   RB              /* ] */
%term   SM              /* ; */
%term   CM              /* , */
%term   TYPE            /* INT CHAR */
%term   EXTERN          /* extern */
%term   WHILE           /* while */
%term   IF              /* if */
%term   ELSE            /* else */
%term   BREAK           /* break */
%term   CONTINUE        /* continue */
%term   RETURN          /* return */

%right  ASG
%left   LOR OR
%left   XOR
%left   LAND AND
%left   LNOT NOT
%left   EQUOP
%left   RELOP
%left   SHIFTOP
%left   PLUS MINUS
%left   MUL DIVOP
%right  ICOP
%left   LB LP

%start  program

%%

program     : defs { 
                  print_name_table(&global_table); 
                  genasm_generate_global_definitions();
                  genasm_generate_string_definitions();
              }
            ;
defs        : defs def { extern_def = 0; }
            |
            ;
def         : extdef def1
            | def1
            ;
extdef      : EXTERN { extern_def = 1; }
            ;
def1        : data_def
            | fnct_def
            | error { release_all_trees(); }
            ;
data_def    : type dcls SM
            ;
type        : TYPE { set_type(current_type, $1.int_value); }
            ;
dcls        : dcls CM dcl
            | dcl
            ;
dcl         : MUL dcl1 {
                  $$ = $2;
                  $$.name->typ.decl |= DECL_PNT;
              }
            | dcl1 { $$ = $1; }
            ;
dcl1        : fun_header { 
                  $$ = $1; 
                  /* Restore current_function; */
                  current_function = (name_ptr) $1.ptr_value;
              }
            | array { $$ = $1; }
            | var { $$ = $1; }
            ;

/* Function header -- used for both prototypes and function definitions */
fun_header  : NAME { 
                  /* Save current_function and current_table */
                  $$.ptr_value = (void *) current_function;
                  $$.ptr_value2 = (void *) current_table;
                  $$.name = process_function_name($1.text_value); 
              }
              LP pars RP { 
                  current_type = &base_type; 
                  /* Restore current_table */
                  current_table = (name_table_ptr) $2.ptr_value2;
                  $$ = $2;
              }
            ;
/* Parameters of function */
pars        : /* empty */
            | pdefs
            ;
pdefs       : pdef
            | pdefs CM pdef
            ;
pdef        : type pdcl { 
                  current_function->typ.num_arguments++;
                  $$ = $2;
              }
            ;
pdcl        : MUL pdcl1 {
                  $$ = $2;
                  $$.name->typ.decl |= DECL_PNT;
              }
            | pdcl1 { $$ = $1; }
            ;
pdcl1       : array { $$ = $1; }
            | var { $$ = $1; }
            ;
array       : var LB dim RB {
                  $$ = $1;
                  $$.name->typ.decl |= DECL_ARR;
                  $$.name->typ.dimension = $3.int_value;
                  assign_type($$.name, current_type);
              }
            ;
var         : NAME { $$.name = process_variable($1.text_value); }
            ;
dim         : ICON { $$ = $1; }
            | /* empty */ { $$.int_value = 0; }
            ;

fnct_def    : type
              fun_head
              LC {
                  current_table = &local_table;
                  copy_params_to_local_table($2.name);
              }
              ldefs {
                  process_local_table();
                  gen_function_beg();
                  break_label = 0; continue_label = 0;
              }
              stmts RC {
                  gen_function_end();
                  release_rtl_function();
                  release_name_table(&local_table);
                  current_table = &global_table;
                  release_all_trees();
              }
            ;
fun_head    : fun_header { $$ = $1; }
            | MUL fun_header {
                  $$ = $2;
                  $$.name->typ.decl |= DECL_PNT;
              }
            ;
ldefs       : ldefs data_def
            |
            ;
stmts       : stmts stmt
            |
            ;
stmt        : ifh stmt { gen_label($1.int_value); }
            | ifelh stmt { gen_label($1.int_value); }
            | whileh stmt {
                  gen_goto(continue_label);
                  gen_label(break_label);
                  break_label = $1.int_value;       /* Restore previous */
                  continue_label = $1.int_value2;   /*           values */
              }
            | RETURN SM         { gen_return(NULL); }
            | RETURN e SM       { 
                  print_tree($2.t);
                  if (expr_type($2.t) >= 0)
                      gen_return($2.t);
                  else
                      gen_return(NULL);
                  release_tree($1.t);
              }
            | CONTINUE SM       {
                  if (continue_label != 0) gen_goto(continue_label);
                  else error_message0("\"continue\" out of a loop body");
              }
            | BREAK SM          { 
                  if (break_label != 0) gen_goto(break_label);
                  else error_message0("\"break\" out of a loop body");
              }
            | LC stmts RC
            | e SM { 
                  print_tree($1.t); 
                  if (expr_type($1.t) >= 0)
                      gen_expr($1.t, 0);
                  release_tree($1.t);
              }
            | SM
            | error SM { release_all_trees(); }
            | error RC { release_all_trees(); }
            ;
ifh         : IF LP e RP {
                  int falseLabel;
                  print_tree($3.t);
                  falseLabel = gen_getlabel();
                  if (expr_type($3.t) >= 0)
                      gen_lexpr($3.t, 0, falseLabel);
                  $$.int_value = falseLabel;
              }
            ;
ifelh       : ifh stmt ELSE {
                  int falseLabel = $1.int_value;
                  int trueLabel = gen_getlabel();
                  gen_goto(trueLabel);
                  gen_label(falseLabel);
                  $$.int_value = trueLabel;
              }
            ;
whileh      : WHILE LP e RP {
                  print_tree($3.t);
                  $$.int_value = break_label;   /* Save previous values */
                  $$.int_value2 = continue_label;
                  break_label = gen_getlabel();
                  continue_label = gen_getlabel();
                  gen_label(continue_label);
                  if (expr_type($3.t) >= 0)
                      gen_lexpr($3.t, 0, break_label);
              }
            ;
e           : e ASG e {
                  tree_ptr node;
                  LBinOp:
                  node = allocate_tree_node();
                  node->oper = $2.int_value;
                  node->left = $1.t; node->right = $3.t;
                  $$.t = node;
              }
            | e LOR e   { goto LBinOp; }
            | e LAND e  { goto LBinOp; }
            | LNOT e    {
                  tree_ptr node;
                  LUnOp: ;
                  node = allocate_tree_node();
                  node->oper = $1.int_value;
                  node->left = $2.t;
                  $$.t = node;
              }
            | e OR e        { goto LBinOp; }
            | e XOR e       { goto LBinOp; }
            | e AND e       { goto LBinOp; }
            | e EQUOP e     { goto LBinOp; }
            | e RELOP e     { goto LBinOp; }
            | e SHIFTOP e   { goto LBinOp; }
            | e PLUS e      { goto LBinOp; }
            | e MINUS e     { goto LBinOp; }
            | e MUL e       { goto LBinOp; }
            | e DIVOP e     { goto LBinOp; }
            | term
            ;
term        : term ICOP     {
                  tree_ptr node = allocate_tree_node();
                  node->oper = $2.int_value;
                  node->left = $1.t;
                  node->modifiers |= MOD_POSTFIX;
                  $$.t = node;
              }
            | NOT term      { goto LUnOp; }
            | MINUS term    { $1.int_value = UNMINUS; goto LUnOp; }
            | MUL term      { $1.int_value = PNT; goto LUnOp; }
            | AND term      { $1.int_value = AMP; goto LUnOp; }
            | ICOP term     { goto LUnOp; }
            | NAME LP elist RP {
                  tree_ptr node = allocate_tree_node();
                  node->oper = FUN;
                  node->name = process_function_call($1.text_value);
                  node->left = $3.t;
                  $$.t = node;
              }
            | NAME LB e RB {
                  /* Process array element "a[e]" as "*(a+e)" */
                  name_ptr n;
                  tree_ptr node0 = allocate_tree_node();
                  tree_ptr node1 = allocate_tree_node();
                  tree_ptr node2 = allocate_tree_node();
                  node0->oper = PNT; node0->left = node1;
                  node1->oper = PLUS;
                  node1->left = node2; node1->right = $3.t;
                  n = name_lookup($1.text_value, &local_table);
                  if (n == NULL)
                      n = name_lookup($1.text_value, &global_table);
                  if (n == NULL)
                      error_message1("Undefined name", $1.text_value);
                  node2->oper = NAME;
                  node2->name = n;
                  $$.t = node0;
              }
            | NAME {
                  name_ptr n;
                  tree_ptr node = allocate_tree_node();
                  node->oper = NAME;
                  n = name_lookup($1.text_value, &local_table);
                  if (n == NULL)
                      n = name_lookup($1.text_value, &global_table);
                  if (n == NULL)
                      error_message1("Undefined name", $1.text_value);
                  node->name = n;
                  $$.t = node;
              }
            | ICON {
                  tree_ptr node = allocate_tree_node();
                  node->oper = ICON;
                  node->int_value = $1.int_value;
                  $$.t = node;
              }
            | STRING {
                  tree_ptr node = allocate_tree_node();
                  node->oper = STRING;
                  node->string_value = addString(strbuf);
                  $$.t = node;
              }
            | LP e RP { $$ = $2; }
            ;
elist       : elist2        { $$ = $1; }
            |               { $$.t = NULL; }
            ;
elist2      : elist2 CM e {
                  tree_ptr node = allocate_tree_node();
                  node->oper = LST;
                  node->int_value = $1.t->int_value + 1; /* Num. args */
                  /* Arguments in reverse order */
                  node->left = $3.t;
                  node->right = $1.t;
                  $$.t = node;
              }
            | e {
                  tree_ptr node = allocate_tree_node();
                  node->oper = LST;
                  node->left = $1.t;
                  node->int_value = 1;  /* number of elements in list */
                  $$.t = node;
              }
            ;
%%

int main(int argc, char *argv[]) {
    parse_command_line(argc, argv);
    init_compiler();
    yyparse();
    finalize_compiler();
    return error_level;
}

int yyerror(const char *s) {
    printf("%s\n", s);
    return 0;
}

void init_compiler() {
    init_name_tables();
    init_trees();
    init_string_list();
    init_rtl();
    genasm_init();
}

void finalize_compiler() {
    genasm_terminate();
    if (yyin != NULL && yyin != stdin)
        fclose(yyin);
    if (dump_file != NULL && dump_file != stdout)
        fclose(dump_file);
    release_string_list();
}
