/*
 * Generate RTL code by expression tree
 */
#ifndef RTLGEN_H
#define RTLGEN_H

#include "tree.h"
#include "nametab.h"
#include "rtl.h"

int gen_expr(                       /* Generate code for ariphm. expr.  */
    tree_ptr expr_tree,             /* In: level = level of recursion   */
    int level                       /*     Returns register with result */
);                                  /*     or (-1) in case of error     */
void gen_lexpr(                     /* Generate code for logical expr.  */
    tree_ptr expr_tree,
    int trueLabel, int falseLabel   /* trueLabel == 0 || falseLabel == 0 */
);
int gen_l2int(tree_ptr expr_tree);  /* Evaluate logical expression */

int gen_getlabel();                 /* Generate label */
int gen_getregister();              /* Generate pseudo-register */
int gen_lastregister();             /* Last pseudo-register */
void gen_init_register_counter();

void gen_start_function();
void gen_label(int label);
void gen_goto(int label);
void gen_condgoto(int cond, int label);
void gen_return(tree_ptr expr);
void gen_setreg(int reg, int val);      /* Generate reg = val */
void gen_setregstr(int reg, string_ptr str);    /* reg = string addr. */
void gen_setregrtx(int reg, rtx_ptr expr);      /* reg = RTL expression */
void gen_sethardregrtx(int hreg, rtx_ptr expr); /* hard_reg = RTL expression */

int gen_readvar(name_ptr n);            /* Read a simple variable */
int gen_binop(tree_ptr e);              /* Perform a binary operation */
int gen_unop(tree_ptr e);               /* Perform an unary operation */
int gen_incrop(tree_ptr e, int level);  /* Increment/decrement */
int gen_assign(tree_ptr e, int level);  /* e1 = e2 */
int gen_tst(tree_ptr e);                /* cc0 = e - 0 */
int gen_cmp(tree_ptr e1, tree_ptr e2);  /* cc0 = e1 - e2 */
int gen_funcall(tree_ptr e, int level); /* Function call */

rtx_ptr gen_rtxreg(int pseudoreg);      /* RTX for a pseudo register */
rtx_ptr gen_rtxhardreg(int hreg);       /* RTX for a hard register */
rtx_ptr gen_rtxcc0();                   /* RTX for CC0 */
rtx_ptr gen_rtxconst(int c);            /* RTX for an integer constant */
rtx_ptr gen_rtxstring(string_ptr s);    /* RTX for an integer constant */
rtx_ptr gen_rtxvar(name_ptr n);         /* RTX for a simple variable */
rtx_ptr gen_rtxvaraddr(name_ptr n);     /* RTX for an addr. of variable */

int check_lvalue(tree_ptr expr);
rtx_ptr gen_rtxderef(tree_ptr e);       /* RTX for dereference */
rtx_ptr gen_rtxaddr(tree_ptr e);        /* RTX for object address */
rtx_ptr gen_rtxlval(tree_ptr e);        /* RTX for object */

#endif /* RTLGEN_H */
