%{
/* Scanner tor Tinyc parser */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "lscan.h"

extern YYSTYPE yylval;

static void scanError(const char *format, int c);

%}

%Start   S COM STR DEF1 DEF2

LETTER    [A-Z_a-z]
NOLETTER [^A-Z_a-z]
DIGIT    [0-9]
IDENT    {LETTER}({LETTER}|{DIGIT})*
NUMBER   {DIGIT}+
WHISP1   [ \t\r]
WHISP    ({WHISP1}|\n)

%%  
                { BEGIN S; }

<S>"\n"         { yylval.int_value = ENDL; return ENDL; }

<S>{IDENT}      { strncpy(yylval.text_value, yytext, NAMEMAX);
                  return( NAME ); }
<S>{NUMBER}     { yylval.int_value = atoi(yytext); return( ICON ); }
<S>"!"          { return( LNOT ); }
<S>"&&"         { return( LAND ); }
<S>"||"         { return( LOR ); }
<S>"<"          { yylval.int_value = LT; return( RELOP ); }
<S>"<="         { yylval.int_value = LE; return( RELOP ); }
<S>">"          { yylval.int_value = GT; return( RELOP ); }
<S>">="         { yylval.int_value = GE; return( RELOP ); }
<S>"=="         { yylval.int_value = EQ; return( EQUOP ); }
<S>"!="         { yylval.int_value = NE; return( EQUOP ); }
<S>"("          { return( LP ); }
<S>")"          { return( RP ); }

<S>{WHISP1}+    { }
<S>.            { scanError( "Bad character %c\n", yytext[0] ); }

%%

int yywrap() { return( 1 ); }

static void scanError(const char *format, int c) {
    printf(format, c);
}
