#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "ltree.h"

static const char* node_type(tree* t);

void release_tree(tree* t) {
    if (t == NULL)
        return;
    if (t->right != NULL)
        release_tree(t->right);
    if (t->left != NULL)
        release_tree(t->left);
    release_node(t);
}

tree* allocate_node() {
    tree* t = (tree*) malloc(sizeof(tree));
    memset(t, 0, sizeof(tree));
    return t;
}

void release_node(tree *node) {
    if (node != NULL)
        free(node);
}

void print_tree(tree* t) {
    if (t == NULL)
        return;

    if (t->type == NAME_NODE) {
        printf("%x %s \"%s\" left=%x right=%x\n",
            t, node_type(t), t->name, t->left, t->right);
    } else if (t->type == ICON_NODE) {
        printf("%x %s %d left=%x right=%x\n",
            t, node_type(t), t->int_value, t->left, t->right);
    } else {
        printf("%x %s left=%x right=%x\n",
            t, node_type(t), t->left, t->right);
    }

    if (t->left != NULL)
        print_tree(t->left);
    if (t->right != NULL)
        print_tree(t->right);
}

static const char* node_type(tree* t) {
    const char *type = "NULL";
    if (t == NULL)
        return type;
    switch (t->type) {
        case NAME_NODE: type = "NAME_NODE"; break;
        case ICON_NODE: type = "ICON_NODE"; break;
        case EXPR_NODE: type = "EXPR_NODE"; break;
        case LNOT_NODE: type = "LNOT_NODE"; break;
        case LAND_NODE: type = "LAND_NODE"; break;
        case LOR_NODE: type = "LOR_NODE"; break;
        case EQUOP_NODE: type = "EQUOP_NODE"; break;
        case RELOP_NODE: type = "RELOP_NODE"; break;
    };
    return type;
}

static char value[256];
const char* node_value(tree *t) {
    value[0] = 0;
    if (t->type == NAME_NODE)
        return t->name;
    if (t->type == ICON_NODE)
        sprintf(value, "%d\0", t->int_value);
    return value;
}
