/*
 * RTL representation
 */
#ifndef RTL_H
#define RTL_H

#include "list.h"
#include "nametab.h"
#include "tree.h"

/* Registers */
/* Must be defined in processor description
#define MIN_PSEUDOREG   22
#define PC              16
#define SP              7
#define FP              6
#define CC0             17
*/
#include "target.h"

/*
 * RTL is an L2-list of elements (or "lines").
 * Every line contains a number of fixed fields and
 * an expression tree for instructions.
 */

/* RTL expression */
typedef struct rtx_tag {
    struct rtx_tag *left;
    struct rtx_tag *right;
    int oper;                   /* mem, reg, operation, etc. */
    int num_args;               /* number of arguments */
    int mem_class;              /* int or char */
    int int_value;              /* integer constant */
    name_ptr local_name;        /* pointer to local table */
    name_ptr global_name;       /* pointer to global table */
    const char *string_value;   /* pointer to a string */
    int modifiers;              /* various flags */
    int reg;                    /* pseudo register */
    int hard_reg;               /* hard register */
    int mem_slot;               /* memory slot */
} rtx, *rtx_ptr;

/* RTL expressions */
#define RTX_NONE            0
#define RTX_REG             1
#define RTX_HARDREG         2
#define RTX_MEM             3
#define RTX_NAME            4
#define RTX_ICON            5
#define RTX_SET             6
#define RTX_PLUS            7
#define RTX_MINUS           8
#define RTX_MUL             9
#define RTX_DIV             10
#define RTX_MOD             11
#define RTX_AND             12
#define RTX_OR              13
#define RTX_XOR             14
#define RTX_CMP             15
#define RTX_TST             16
#define RTX_LSHIFT          17
#define RTX_RSHIFT          18
#define RTX_INC             19
#define RTX_DEC             20
#define RTX_CALL            21
#define RTX_LABEL           22
#define RTX_IF_THEN_ELSE    23
#define RTX_PREDEC          24
#define RTX_POSTINC         25
#define RTX_UNMINUS         26
#define RTX_NOT             27

/* Bits in rtx.modifiers -- the same as defined in "tree.h" */
#define RTX_MODPOSTFIX  MOD_POSTFIX     /* Postfix operation ++, -- */
#define RTX_MODBYTE     MOD_BYTE        /* Byte instruction */

rtx_ptr allocate_rtx();
rtx_ptr clone_rtx(rtx_ptr expr);
void release_rtx(rtx_ptr expr);

/* RTL line */
typedef struct rtl_tag {
    block_header link;          /* link to next/prev lines */
    int typ;                    /* type of line */
    int int_value;              /* int value for labels, etc */
    rtx_ptr expr;               /* rtl expression */
    int int_value2;             /* int value for conditions, etc. */
} rtl, *rtl_ptr;

/* Types of RTL lines */
#define RTL_FUNCTION_BEG        1
#define RTL_FUNCTION_END        2
#define RTL_LABEL               3
#define RTL_GOTO                4
#define RTL_USE                 5
#define RTL_INSTR               6

void init_rtl();
rtl_ptr allocate_rtl_line();
void release_rtl_line(rtl_ptr line);
void release_rtl_function();
void add_rtl_line(rtl_ptr line);
void add_rtl_label(int l);
void add_rtl_goto(int l);
void add_rtl_cond_goto(int condition, int lab);
void add_rtl_function_beg();
void add_rtl_function_end();

void print_rtl_function();
void print_rtx(rtx_ptr expr);

const char *cond_name(int condition);
int inverse_cond(int condition);

extern block_header rtl_function;   /* rtl of current function */

#endif /* RTL_H */
